@@ -42,6 +42,8 @@ grad_avp_t *radscm_list_to_avl(SCM list);
 SCM radscm_avp_to_cons(grad_avp_t *pair);
 grad_avp_t *radscm_cons_to_avp(SCM scm);
 
+static void scheme_before_config_hook(void *a, void *b);
+
 
 /* General-purpose eval handlers */
 
@@ -258,8 +260,13 @@ scheme_eval_boolean_expr(char *expr)
 static SCM
 catch_body(void *data)
 {
+        SCM orig_load_path;
+         
 	scm_init_load_path();
 	grad_scm_init();
+	orig_load_path = RAD_SCM_SYMBOL_VALUE("%load-path");
+	radiusd_set_preconfig_hook(scheme_before_config_hook, orig_load_path,
+				   0);
 	rscm_server_init();
 	scheme_load_module("radiusd");
 	radiusd_main();
@@ -273,6 +280,14 @@ catch_handler(void *data, SCM tag, SCM throw_args)
 }
 
 
+static void
+scheme_before_config_hook(void *data, void *b ARG_UNUSED)
+{
+	SCM *pscm;
+	pscm = SCM_VARIABLE_LOC(scm_c_lookup("%load-path"));
+	*pscm = (SCM) data;
+}
+
 void
 scheme_boot(void *closure, int argc, char **argv)
 {
@@ -281,6 +296,7 @@ scheme_boot(void *closure, int argc, char **argv)
 			   catch_handler, NULL);
 }
 
+
 void
 scheme_main()
 {
@@ -292,7 +308,7 @@ scheme_main()
 /* Entry points for main Radius tasks */
 
 int
-scheme_try_auth(int auth_type, grad_request_t *req,
+scheme_try_auth(int auth_type, radiusd_request_t *req,
 		grad_avp_t *user_check,
 		grad_avp_t **user_reply_ptr)
 {
@@ -300,7 +316,7 @@ scheme_try_auth(int auth_type, grad_request_t *req,
 	SCM res;
 	grad_avp_t *tmp =
 		radius_decrypt_request_pairs(req,
-					     grad_avl_dup(req->request));
+					     grad_avl_dup(req->request->avlist));
         static char *try_auth = "radiusd-try-auth";
 	
 	s_request = radscm_avl_to_list(tmp);
@@ -340,7 +356,7 @@ scheme_try_auth(int auth_type, grad_request_t *req,
 }
 
 int
-scheme_auth(char *procname, grad_request_t *req, 
+scheme_auth(char *procname, radiusd_request_t *req, 
 	    grad_avp_t *user_check,
 	    grad_avp_t **user_reply_ptr)
 {
@@ -348,7 +364,7 @@ scheme_auth(char *procname, grad_request_t *req,
 	SCM res;
 	grad_avp_t *tmp =
 		radius_decrypt_request_pairs(req,
-					     grad_avl_dup(req->request));
+					     grad_avl_dup(req->request->avlist));
 	
 	s_request = radscm_avl_to_list(tmp);
 	radius_destroy_pairs(&tmp);
@@ -384,10 +400,10 @@ scheme_auth(char *procname, grad_request_t *req,
 }
 
 int
-scheme_acct(char *procname, grad_request_t *req)
+scheme_acct(char *procname, radiusd_request_t *req)
 {
 	SCM res;
-	SCM s_request = radscm_avl_to_list(req->request);
+	SCM s_request = radscm_avl_to_list(req->request->avlist);
 
 	if (scheme_call_proc(&res,
 			     procname,
@@ -651,7 +667,7 @@ struct cfg_stmt guile_stmt[] = {
 #include <radius/radius.h>
 
 int
-scheme_try_auth(int auth_type, grad_request_t *req,
+scheme_try_auth(int auth_type, radiusd_request_t *req,
 	        grad_avp_t *user_check,
 	        grad_avp_t **user_reply_ptr)
 {
